@@ -44,7 +44,6 @@
 /* ************************************************************* */
 
 /* program stuff */
-
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/wait.h>
@@ -55,7 +54,11 @@
 #include <curses.h>
 
 #ifndef __FreeBSD__
-# include <crypt.h>
+# ifdef __APPLE__
+#  include <unistd.h>
+# else
+#  include <crypt.h>
+# endif
 #else
 # include <libutil.h>
 #endif
@@ -162,6 +165,24 @@ gen_ttyrec_filename ()
 
 /* ************************************************************* */
 
+char*
+gen_nhext_filename ()
+{
+  time_t rawtime;
+  struct tm *ptm;
+  char *nhext_filename = calloc(100, sizeof(char));
+
+  /* append time to filename */
+  time (&rawtime);
+  ptm = gmtime (&rawtime);
+  snprintf (nhext_filename, 100, "%04i-%02i-%02i.%02i:%02i:%02i.nhext",
+            ptm->tm_year + 1900, ptm->tm_mon + 1, ptm->tm_mday,
+            ptm->tm_hour, ptm->tm_min, ptm->tm_sec);
+  return nhext_filename;
+}
+
+/* ************************************************************* */
+
 char*
 gen_inprogress_lock (pid_t pid, char* ttyrec_filename)
 {
@@ -309,14 +330,23 @@ inprogressmenu ()
   int i, menuchoice, len = 20, offset = 0, doresizewin = 0;
   time_t ctime;
   struct dg_game **games;
-  char ttyrecname[130], *replacestr = NULL;
+  char ttyrecname[130], *replacestr = NULL, gametype[10];
+  int is_nhext[14];
   sigset_t oldmask, toblock;
 
   games = populate_games (&len);
 
   while (1)
     {
-      clear ();
+      if (len == 0)
+        offset = 0;
+      else
+        {
+	  while (offset >= len && offset >= 14)
+	    offset -= 14;
+	}
+
+      erase ();
       drawbanner (1, 1);
       mvprintw (3, 1,
                 "During playback, hit 'q' to return here, 'm' to send mail (requires login),");
@@ -332,9 +362,16 @@ inprogressmenu ()
           if (i + offset >= len)
             break;
 
-          mvprintw (7 + i, 1, "%c) %-15s (%3dx%3d) %s %s (%ldm %lds idle)",
-                    i + 97, games[i + offset]->name,
-		    games[i + offset]->ws_col, games[i + offset]->ws_row,
+	  is_nhext[i] = !strcmp (games[i + offset]->ttyrec_fn + strlen (games[i + offset]->ttyrec_fn) - 6, ".nhext");
+
+	  if (is_nhext[i])
+	    strcpy (gametype, "  NhExt");
+	  else
+	    snprintf (gametype, sizeof gametype, "%3dx%3d",
+		games[i + offset]->ws_col, games[i + offset]->ws_row);
+
+          mvprintw (7 + i, 1, "%c) %-15s (%s) %s %s (%ldm %lds idle)",
+                    i + 97, games[i + offset]->name, gametype,
                     games[i + offset]->date, games[i + offset]->time,
                     (time (&ctime) - games[i + offset]->idle_time) / 60,
                     (time (&ctime) - games[i + offset]->idle_time) % 60);
@@ -374,6 +411,9 @@ inprogressmenu ()
 	    }
           if ((menuchoice - 'a') >= 0 && (menuchoice - 'a') < i)
             {
+	      if (is_nhext[menuchoice - 97]) /* Cannot watch NhExt game */
+		break;
+
               /* valid choice has been made */
               snprintf (ttyrecname, 130, "%sttyrec/%s", myconfig->dglroot,
                         games[menuchoice - 97 + offset]->ttyrec_fn);
@@ -678,7 +718,7 @@ drawmenu ()
       mvaddstr (banner.len + 5, 1, "e) Change email address");
       mvaddstr (banner.len + 6, 1, "o) Edit option file");
       mvaddstr (banner.len + 7, 1, "w) Watch games in progress");
-      mvaddstr (banner.len + 8, 1, "p) Play nethack!");
+      mvprintw (banner.len + 8, 1, "p) Play %s!", myconfig->game_name);
       mvaddstr (banner.len + 9, 1, "q) Quit");
       mvaddstr (banner.len + 11, 1, "=> ");
     }
@@ -745,6 +785,21 @@ initcurses ()
 
 /* ************************************************************* */
 
+void
+autologin (char* user, char *pass)
+{
+  int me_index = -1;
+  if ((me_index = userexist(user)) != -1)
+  {
+    me = users[me_index];
+    if (passwordgood(pass))
+    {
+      loggedin = 1;
+      snprintf (rcfilename, 80, "%srcfiles/%s.nethackrc", myconfig->dglroot, me->username);
+    }
+  }
+}
+
 void
 loginprompt (int from_ttyplay)
 {
@@ -804,12 +859,16 @@ loginprompt (int from_ttyplay)
       loggedin = 1;
       snprintf (rcfilename, 80, "%srcfiles/%s.nethackrc", myconfig->dglroot, me->username);
     }
-  else if (from_ttyplay == 1)
+  else 
   {
-    mvaddstr(9, 1, "Login failed. Returning to game.");
-    refresh();
-    sleep(2);
-  }
+    me = NULL;
+    if (from_ttyplay == 1)
+    {
+      mvaddstr(9, 1, "Login failed. Returning to game.");
+      refresh();
+      sleep(2);
+    }
+  } 
 }
 
 /* ************************************************************* */
@@ -964,8 +1023,7 @@ newuser ()
 int
 passwordgood (char *cpw)
 {
-  if (me == NULL)
-    return 1;
+  assert (me != NULL);
 
   if (!strncmp (crypt (cpw, cpw), me->password, 13))
     return 1;
@@ -1283,8 +1341,103 @@ writefile (int requirenew)
 /* ************************************************************* */
 /* ************************************************************* */
 
-/* TODO: Some of the messages here (sorry no nethack for you!) are nethack specific
- * as may be some code... don't think so though. Globalize it. */ 
+/*
+ * Backup the savefile, if configured.
+ * Returns non-zero if successful, otherwise an error message has been
+ * given already.
+ */
+int
+backup_savefile (void)
+{
+  char buf[1024];
+  char *f, *p, *end;
+  int ispercent = 0, n;
+  int in, out;
+
+  f = myconfig->savefilefmt;
+
+  if (*f == '\0')
+    return 1;
+  if (me == NULL)
+    graceful_exit (147);
+
+  p = buf;
+  end = buf + sizeof(buf) - 10; /* make sure we can add .bak */
+  while (*f)
+    {
+      if (ispercent)
+        {
+          switch (*f)
+  	  {
+  	    case 'u':
+  	      snprintf (p, end + 1 - p, "%d", myconfig->shed_uid);
+  	      while (*p != '\0')
+  	        p++;
+  	      break;
+  	    case 'n':
+  	      snprintf (p, end + 1 - p, "%s", me->username);
+  	      while (*p != '\0')
+  	        p++;
+  	      break;
+  	    default:
+  	      *p = *f;
+  	      if (p < end)
+  	        p++;
+  	  }
+	  ispercent = 0;
+	}
+      else
+        {
+	  if (*f == '%')
+	    ispercent = 1;
+	  else
+	    {
+	      *p = *f;
+	      if (p < end)
+	        p++;
+	    }
+	}
+      f++;
+    }
+  *p = '\0';
+
+  /*fprintf(stderr, "***\n[SAVEFILE=%s]\n***\n", buf);
+  sleep(3);*/
+  in = open (buf, O_RDONLY);
+  if (in == -1)
+    {
+      if (errno == ENOENT)
+        return 1; /* Nothing to back up */
+      else
+        {
+          fprintf (stderr, "Cannot open savefile '%s'\n", buf);
+	  perror ("for input");
+          return 0;
+        }
+    }
+  strcpy (p, ".bak");
+  out = open (buf, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+  if (out == -1)
+    {
+      close (in);
+      fprintf (stderr, "Cannot open backup savefile '%s'\n", buf);
+      perror ("for output");
+      return 0;
+    }
+
+  while ((n = read (in, buf, sizeof(buf))) > 0)
+    {
+      n = write (out, buf, n);
+      if (n < 0)
+        break;
+    }
+  close (out);
+  close (in);
+  if (n < 0)
+    perror ("I/O error while backing up savefile");
+  return n >= 0;
+}
+
 int
 purge_stale_locks (void)
 {
@@ -1345,7 +1498,8 @@ purge_stale_locks (void)
 
 #define HUP_WAIT 10 /* seconds before HUPPING */
 	mvprintw (3, 1,
-	    "There are some stale Nethack processes, will recover in %d  seconds.", HUP_WAIT);
+	    "There are some stale %s processes, will recover in %d  seconds.",
+            myconfig->game_name, HUP_WAIT);
 	mvaddstr (4, 1,
 	    "Press a key NOW if you don't want this to happen!");
 
@@ -1386,8 +1540,8 @@ purge_stale_locks (void)
           sleep (1);
           if (seconds == 10)
             {
-              mvaddstr (3, 1,
-                        "Couldn't terminate one of your stale Nethack processes gracefully.");
+              mvprintw (3, 1,
+                        "Couldn't terminate one of your stale %s processes gracefully.", myconfig->game_name);
               mvaddstr (4, 1, "Force its termination? [yn] ");
               if (tolower (getch ()) == 'y')
                 {
@@ -1397,8 +1551,8 @@ purge_stale_locks (void)
               else
                 {
                   endwin ();
-                  fprintf (stderr, "Sorry, no nethack for you now, please "
-                           "contact the admin.\n");
+                  fprintf (stderr, "Sorry, no %s for you now, please "
+                           "contact the admin.\n", myconfig->game_name);
                   graceful_exit (1);
                 }
             }
@@ -1454,21 +1608,82 @@ menuloop (void)
     }
 }
 
+int
+authenticate ()
+{
+  int i, len, me_index;
+  char user_buf[22], pw_buf[22];
+  struct dg_game **games;
+
+  /* We use simple password authentication, rather than challenge/response. */
+  printf ("\n");
+  fflush(stdout);
+
+  fgets (user_buf, sizeof(user_buf), stdin);
+  len = strlen (user_buf);
+  if (user_buf[len - 1] == '\n')
+    user_buf[--len] = '\0';
+  else
+    {
+      fprintf (stderr, "Username too long (max 20 chars).\n");
+      return 1;
+    }
+
+  fgets (pw_buf, sizeof(pw_buf), stdin);
+  len = strlen (pw_buf);
+  if (pw_buf[len - 1] == '\n')
+    pw_buf[--len] = '\0';
+  else
+    {
+      fprintf (stderr, "Password too long (max 20 chars).\n");
+      return 1;
+    }
+
+  if ((me_index = userexist (user_buf)) != -1)
+    {
+      me = users[me_index];
+      if (passwordgood (pw_buf))
+        {
+	  games = populate_games (&len);
+	  for (i = 0; i < len; i++)
+	    if (!strcmp (games[i]->name, user_buf))
+	      {
+		fprintf (stderr, "Game already in progress.\n");
+		return 1;
+	      }
+	  win.ws_row = win.ws_col = 0;
+	  gen_inprogress_lock (getppid (), gen_nhext_filename ());
+	  return 0;
+	}
+    }
+
+  sleep (2);
+  fprintf (stderr, "Login failed.\n");
+  return 1;
+}
+
 int
 main (int argc, char** argv)
 {
   /* for chroot and program execution */
-  char atrcfilename[81], *spool;
+  char atrcfilename[81], *spool, *auth = getenv("USER");
   unsigned int len;
   int c;
+  int nhext = 0, nhauth = 0;
 
-  while ((c = getopt(argc, argv, "qh:pp:f:")) != -1)
+  while ((c = getopt(argc, argv, "qh:pp:f:ae")) != -1)
   {
     switch (c)
     {
       case 'q':
 	silent = 1; break;
 
+      case 'a':
+	nhauth = 1; break;
+
+      case 'e':
+	nhext = 1; break;
+
       case 'f':
 	if (config)
 	{
@@ -1501,37 +1716,54 @@ main (int argc, char** argv)
     }
 
   /* get master tty just before chroot (lives in /dev) */
-  ttyrec_getpty ();
+  if (!nhext && !nhauth)
+    ttyrec_getpty ();
 
-  /* chroot */
-  if (chroot (myconfig->chroot))
+  if (geteuid () != myconfig->shed_uid)
     {
-      perror ("cannot change root directory");
-      graceful_exit (1);
+      /* chroot */
+      if (chroot (myconfig->chroot))
+	{
+	  perror ("cannot change root directory");
+	  graceful_exit (1);
+	}
+
+      if (chdir ("/"))
+	{
+	  perror ("cannot chdir to root directory");
+	  graceful_exit (1);
+	}
+
+      /* shed privs. this is done immediately after chroot. */
+      if (setgroups (1, &myconfig->shed_gid) == -1)
+	{
+	  perror ("setgroups");
+	  graceful_exit (1);
+	}
+
+      if (setgid (myconfig->shed_gid) == -1)
+	{
+	  perror ("setgid");
+	  graceful_exit (1);
+	}
+
+      if (setuid (myconfig->shed_uid) == -1)
+	{
+	  perror ("setuid");
+	  graceful_exit (1);
+	}
     }
 
-  if (chdir ("/"))
+  if (nhext)
     {
-      perror ("cannot chdir to root directory");
-      graceful_exit (1);
-    }
+      char *myargv[3];
 
-  /* shed privs. this is done immediately after chroot. */
-  if (setgroups (1, &myconfig->shed_gid) == -1)
-    {
-      perror ("setgroups");
-      graceful_exit (1);
-    }
+      myargv[0] = myconfig->game_path;
+      myargv[1] = "--proxy";
+      myargv[2] = 0;
 
-  if (setgid (myconfig->shed_gid) == -1)
-    {
-      perror ("setgid");
-      graceful_exit (1);
-    }
-
-  if (setuid (myconfig->shed_uid) == -1)
-    {
-      perror ("setuid");
+      execvp (myconfig->game_path, myargv);
+      perror (myconfig->game_path);
       graceful_exit (1);
     }
 
@@ -1539,6 +1771,28 @@ main (int argc, char** argv)
   if (readfile (0))
     graceful_exit (110);
 
+  if (nhauth)
+    graceful_exit (authenticate ());
+
+  if (auth)
+  {
+    char *user, *pass, *p;
+
+    p = strchr(auth, ':');
+
+    if (p)
+    {
+      pass = p + 1;
+
+      if (*pass != '\0')
+      {
+        *p = '\0';
+        user = auth;
+        autologin(user, pass);
+      }
+    }
+  }
+  
   initcurses ();
   menuloop();
 
@@ -1550,6 +1804,9 @@ main (int argc, char** argv)
   endwin ();
   signal(SIGWINCH, SIG_DFL);
 
+  if (!backup_savefile ())
+    graceful_exit (5);
+
   /* environment */
   snprintf (atrcfilename, 81, "@%s", rcfilename);
 
