@@ -45,7 +45,8 @@ struct dg_game
 struct dg_config
 {
   char* chroot;
-  char* nethack;
+  char* game_path;
+  char* game_name;
   char* dglroot;
   char* lockfile;
   char* passwd;
@@ -57,6 +58,7 @@ struct dg_config
   uid_t shed_uid;
   gid_t shed_gid;
   unsigned long max;
+  char* savefilefmt;
 };
 
 /* Global variables */
@@ -85,6 +87,7 @@ extern void freefile(void);
 extern void initcurses(void);
 extern void loginprompt(int from_ttyplay);
 extern void newuser(void);
+extern void autologin(char *user, char *pass);
 extern int passwordgood(char *cpw);
 extern int readfile(int nolock);
 extern int userexist(char *cname);
