@@ -7,6 +7,7 @@
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
+#include <signal.h>
 #include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
@@ -18,7 +19,8 @@ extern int yyparse ();
 struct dg_config *myconfig = NULL;
 struct dg_config defconfig = {
   /* chroot = */ "/var/lib/dgamelaunch/",
-  /* nethack = */ "/bin/nethack",
+  /* game_path = */ "/bin/nethack",
+  /* game_name = */ "NetHack",
   /* dglroot = */  "/dgldir/",
   /* lockfile = */ "/dgl-lock",
   /* passwd = */ "/dgl-login",
@@ -29,7 +31,8 @@ struct dg_config defconfig = {
   /* shed_group = */ "games",
   /* shed_uid = */ 5,
   /* shed_gid = */ 60, /* games:games in Debian */
-  /* max = */ 64000 
+  /* max = */ 64000,
+  /* savefilefmt = */ "" /* don't do this by default */
 };
 
 char* config = NULL;
@@ -41,7 +44,7 @@ char *chosen_name;
 struct dg_game **
 populate_games (int *l)
 {
-  int fd, len, n;
+  int fd, len, n, is_nhext, pid;
   DIR *pdir;
   struct dirent *pdirent;
   struct stat pstat;
@@ -70,22 +73,27 @@ populate_games (int *l)
       if (!strcmp (pdirent->d_name, ".") || !strcmp (pdirent->d_name, ".."))
         continue;
 
+      is_nhext = !strcmp (pdirent->d_name + strlen (pdirent->d_name) - 6, ".nhext");
+
       snprintf (fullname, 130, "%sinprogress/%s", myconfig->dglroot, pdirent->d_name);
 
       fd = 0;
       /* O_RDWR here should be O_RDONLY, but we need to test for
        * an exclusive lock */
       fd = open (fullname, O_RDWR);
-      if ((fd > 0) && fcntl (fd, F_SETLK, &fl) == -1)
+      if (fd >= 0 && (is_nhext || fcntl (fd, F_SETLK, &fl) == -1))
         {
 
           /* stat to check idle status */
-          snprintf (ttyrecname, 130, "%sttyrec/%s", myconfig->dglroot, pdirent->d_name);
-          replacestr = strchr (ttyrecname, ':');
-          if (!replacestr)
-            graceful_exit (145);
-          replacestr[0] = '/';
-          if (!stat (ttyrecname, &pstat))
+	  if (!is_nhext)
+	    {
+	      snprintf (ttyrecname, 130, "%sttyrec/%s", myconfig->dglroot, pdirent->d_name);
+	      replacestr = strchr (ttyrecname, ':');
+	      if (!replacestr)
+		graceful_exit (145);
+	      replacestr[0] = '/';
+	    }
+          if (is_nhext || !stat (ttyrecname, &pstat))
             {
               /* now it's a valid game for sure */
               games = realloc (games, sizeof (struct dg_game) * (len + 1));
@@ -117,6 +125,7 @@ populate_games (int *l)
 		}
 	      else
 		p = "";
+	      pid = atoi(p);
 	      while (*p != '\0' && *p != '\n')
 	        p++;
 	      if (*p != '\0')
@@ -127,13 +136,30 @@ populate_games (int *l)
 	      if (*p != '\0')
 	        p++;
 	      games[len]->ws_col = atoi(p);
-	      if (games[len]->ws_row < 4 || games[len]->ws_col < 4)
+	      if (is_nhext)
 	        {
-		  games[len]->ws_row = 24;
-		  games[len]->ws_col = 80;
-	        }
-
-              len++;
+		  if (kill (pid, 0) != 0)
+		    {
+		      /* Dead game */
+		      free (games[len]->ttyrec_fn);
+		      free (games[len]->name);
+		      free (games[len]->date);
+		      free (games[len]->time);
+		      free (games[len]);
+		      unlink (fullname);
+		    }
+		  else
+		    len++;
+		}
+	      else
+	        {
+		  if (games[len]->ws_row < 4 || games[len]->ws_col < 4)
+		  {
+		    games[len]->ws_row = 24;
+		    games[len]->ws_col = 80;
+		  }
+		  len++;
+		}
             }
         }
       else
@@ -231,10 +257,12 @@ create_config ()
   if (myconfig->max == 0 && !set_max) myconfig->max = defconfig.max;
   if (!myconfig->banner) myconfig->banner = defconfig.banner;
   if (!myconfig->chroot) myconfig->chroot = defconfig.chroot;
-  if (!myconfig->nethack) myconfig->nethack = defconfig.nethack;
+  if (!myconfig->game_path) myconfig->game_path = defconfig.game_path;
+  if (!myconfig->game_name) myconfig->game_name = defconfig.game_name;
   if (!myconfig->dglroot) myconfig->dglroot = defconfig.dglroot;
   if (!myconfig->rcfile) myconfig->rcfile = defconfig.rcfile;
   if (!myconfig->spool) myconfig->spool = defconfig.spool;
   if (!myconfig->passwd) myconfig->passwd = defconfig.passwd;
   if (!myconfig->lockfile) myconfig->lockfile = defconfig.lockfile;
+  if (!myconfig->savefilefmt) myconfig->savefilefmt = defconfig.savefilefmt;
 }
